﻿/**
    @fileOverview
    针对手机网站的阅读模式
    @author lijunwei <lijunwei@maxthon.net>
 */




(function (window) {
    var page;
    var code;

    // 常用函数变量,为了兼容旧有代码
    var getRect = readerTool.getRect;
    var getStyle = readerTool.getStyle;
    var getFontIntSize = readerTool.getFontIntSize;
    var deleteBlank = readerTool.deleteBlank;
    var isVisible = readerTool.isVisible;
    var compareUrlHost = readerTool.compareUrlHost;
    var unicodeHit = readerTool.unicodeHit;
    var stringTail2Number = readerTool.stringTail2Number;
    var isLikelyUrl = readerTool.isLikelyUrl;
    var getDefaultScrollWidth = readerTool.getDefaultScrollWidth;
    var findSameParent = readerTool.findSameParent;

    window.reader = {
        maxthonReaderLaunch: maxthonReaderLaunch,
        getTitle: getTitle,
        getContent: getContent,
        getNextUrl: getNextUrl
    };
    
    function getTitle() {
        return page.getArticleTitle();
    }
    
    function getContent() {
        return page.getArticleContent();
    }    

    function getNextUrl() {
        return page.getNextPageUrl();
    }

    function maxthonReaderLaunch(doc) {
        code = isWebUrlList(window.location.host);
        if (code > -1) {
            page = new PageConstructor(doc);
        }
        if (page && page.isRead()) {
            return true;
        }
        return false;
    }
    
    /**
        @description 页面构造分析
        @param document
        @return true代表可以启动阅读模式
     */
    function PageConstructor(doc) {
        this.articleTitle;
        this.articleContent;
        this.nextPageUrl;

        this.isRead = function () {
            this.getArticleTitle();
            this.getArticleContent();
            if (this.articleTitle && this.articleContent) {
                return true;
            }
            return false;
        };
        
        //返回文字title
        this.getArticleTitle = function () {
            var htmlTitle;
            if (this.articleTitle) {
                return this.articleTitle;
            }
            switch (code) {
                case 11: //红袖
                    htmlTitle = doc.querySelector('body > strong').innerHTML;
                    break;
                case 20: //Business Insider
                    htmlTitle = doc.querySelector('.instapaper_title').innerHTML;
                    break;
                case 21: //Washington Post
                    htmlTitle = doc.querySelector('.content > .article-title').innerHTML;
                    break;
                case 23: //CNN The Marquee Blog
                    htmlTitle = doc.querySelector('.cnnm-title > a').innerHTML;
                    break;
                case 24: //Fox News
                    htmlTitle = doc.querySelector('.element > b').innerHTML;
                    break;
                case 25: //Huffington Post
                    htmlTitle = doc.querySelector('.entryTitle').innerHTML;
                    break;
                case 26: //BBC
                    htmlTitle = doc.querySelector('.story-body h1').innerHTML;
                    break;
                case 30: //Engadget
                    htmlTitle = doc.querySelector('.paddingTop h2').innerHTML;
                    break;
                case 32: //Mac Rumors
                    htmlTitle = doc.querySelector('.teaser_title').innerHTML;
                    break;
                default :
                    htmlTitle = doc.querySelector('title').innerHTML;
            }
            this.articleTitle = htmlTitle ? htmlTitle : '';
        }

        //返回内容
        this.getArticleContent = function() {
            var htmlObj;
            var htmlStr;
            if (this.articleContent) {
                return this.articleContent;
            }
            switch (code) {
                case 0: //网易
                    htmlObj = doc.querySelector('.text > .text1');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 1: //新浪读书
                    htmlObj = doc.querySelector('#zwbox > .blue_box');
                    if (!htmlObj) {
                        htmlObj = doc.querySelector('.wrap > *:nth-child(9)');
                    }
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 2: //新浪
                    htmlObj = doc.querySelector('.mainbox > #content');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchSectionToNul(htmlStr, '<script', 1);
                    break;
                case 3: //腾讯读书
                    htmlObj = doc.querySelector('.mqq-content > .mqq-p');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 4: //腾讯
                    htmlObj = doc.querySelector('#articlecontent');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 5: //凤凰读书
                    htmlObj = doc.querySelector('.listM > .content');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = /马上阅读/.test(htmlStr) ? '' : matchSectionToNul(htmlStr, '<form', 1);
                    htmlStr = matchTagToNull(htmlStr, 'div', 1);
                    htmlStr = matchTagToNull(htmlStr, 'a' ,0);
                    break;
                case 6: //凤凰
                    htmlObj = doc.querySelector('#articlecontent');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 7: //3G门户
                    htmlObj = doc.querySelector('.content_text');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 8: //3G门户书城
                    htmlObj = doc.querySelector('#body0');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchSectionToNul(htmlStr, '<input', 1);
                    htmlStr = matchTagToNull(htmlStr, 'a' ,0);
                    break;
                case 9: //搜狐(旧版)
                    htmlObj = doc.querySelectorAll('#newsimg');
                    htmlStr = objToStr(htmlObj);
                    htmlObj = doc.querySelector('#newscontet');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 10: //搜狐(新版)
                    htmlObj = doc.querySelector('.finCnt .cnt');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchTagToNull(htmlStr, 'a' ,0);
                    break;
                case 11: //红袖
                    htmlObj = doc.querySelector('#htmlContent');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchStrToNull(htmlStr, '&nbsp;');
                    break;
                case 12: //人民网
                    htmlObj = doc.querySelectorAll('font[class="c7"]');
                    htmlStr = objToStr(htmlObj);
                    break;
                case 13: //新华网
                    htmlObj = doc.querySelector('.content');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 14: //铁血网
                    htmlObj = doc.querySelector('.text_content');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 15: //环球网
                    htmlObj = doc.querySelector('#content');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 16: //起点
                    htmlObj = doc.querySelectorAll('.articleDes > h2');
                    htmlStr = objToStr(htmlObj);
                    htmlStr = /阅读设置/.test(htmlStr) ? matchSectionToNul(htmlStr, '阅读设置', 1) : '';
                    htmlStr = matchTagToNull(htmlStr, 'h1' ,0);
                    htmlStr = matchTagToNull(htmlStr, 'a' ,0);
                    break;
                case 17: //塔读网
                    htmlObj = doc.querySelector('.my_style');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchTagToNull(htmlStr, 'span' ,0);
                    break;
                case 18: //晋江原创网
                    htmlObj = doc.querySelector('ul');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 19: //Yahoo News
                    htmlObj = doc.querySelector('#content > div > .uic.center');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlObj = doc.querySelector('#content > div > .uic.description');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 20: //Business Insider
                    htmlObj = doc.querySelector('.instapaper_body');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchSectionToNul(htmlStr, '<div\\sclass="slide-controls', 1);
                    break;
                case 21: //Washington Post
                    htmlObj = doc.querySelector('.article_image');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlObj = doc.querySelector('.article_caption');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    htmlObj = doc.querySelector('.article_body_text');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchSectionToNul(htmlStr, '<div\\sclass="wpd_pagexofy', 1);
                    break;
                case 22: //CNN
                    htmlObj = doc.querySelector('.articleContent');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchTagToNull(htmlStr, 'span', 0, 'class="bold_text"');
                    htmlStr = matchTagToNull(htmlStr, 'div', 0);
                    htmlStr = matchTagToNull(htmlStr, 'script', 0);
                    break;
                case 23: //CNN The Marquee Blog
                    htmlObj = doc.querySelector('.cnnm-wideimg');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlObj = doc.querySelector('.cnnm-post-body');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 24: //Fox News
                    htmlObj = doc.querySelectorAll('div[class="element"] > p');
                    htmlStr = objToStr(htmlObj);
                    break;
                case 25: //Huffington Post
                    htmlObj = doc.querySelector('.entryVideo');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlObj = doc.querySelector('.entryImage');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    htmlObj = doc.querySelector('.entryBody');
                    htmlStr += htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 26: //BBC
                    htmlObj = doc.querySelector('.story-body');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    htmlStr = matchTagToNull(htmlStr, 'div', 0, 'class="byline"');
                    htmlStr = matchTagToNull(htmlStr, 'h1', 0);
                    break;
                case 27: //BBC Sport
                    htmlObj = doc.querySelector('.storybody');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 28: //TC
                    htmlObj = doc.querySelector('.f_article > span');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 29: //GigaOM
                    htmlObj = doc.querySelector('.content > .post');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 30: //Engadget
                    htmlObj = doc.querySelector('.paddGen > span');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 31: //ESPN
                    htmlObj = doc.querySelector('#story-body');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 32: //Mac Rumors
                    htmlObj = doc.querySelector('.teaser_content');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 33: //NYT
                    htmlObj = doc.querySelector('.bodycontent');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
                case 34: //新浪博客
                    htmlObj = doc.querySelector('.lS4');
                    htmlStr = htmlObj ? htmlObj.innerHTML : '';
                    break;
            }
            this.articleContent = filterHtml(htmlStr);
        }

        //获取下一页链接
        this.getNextPageUrl = function() {
            var urlHref;
            if (this.nextPageUrl) {
                return this.nextPageUrl;
            }
            switch (code) {
                case 2: //新浪
                    urlHref = findNextPageUrl(doc, '余下全文');
                    this.nextPageUrl = urlHref ? urlHref : findNextPageUrl(doc, '下一页|下页');
                    break;
                case 6: //凤凰
                    this.nextPageUrl = findNextPageUrl(doc, '<input\\sclass="nextpage');
                    break;
                case 13: //新华网
                    this.nextPageUrl = findNextPageUrl(doc, '^&gt;$');
                    break;
                case 14: //铁血网
                    this.nextPageUrl = null;
                    break;
                case 20: //Business Insider
                    this.nextPageUrl = findNextPageUrl(doc, '^Next\\s»');
                    break;
                case 21: //Washington Post
                    this.nextPageUrl = findNextPageUrl(doc, '^Next\\sPage');
                    break;
                case 22: //CNN
                    this.nextPageUrl = findNextPageUrlSpecial1();
                    break;
                case 24: //Fox News
                    this.nextPageUrl = findNextPageUrl(doc, '^Next\\sPage');
                    break;
                case 34: //新浪博客
                    urlHref = findNextPageUrl(doc, '余下全文');
                    this.nextPageUrl = urlHref ? urlHref : findNextPageUrl(doc, '下一页|下页');
                    break;
                default :
                    urlHref = findNextPageUrl(doc, '下一页|下页');
                    this.nextPageUrl = urlHref ? urlHref : findNextPageUrl(doc, '下一章|下章|下一节');
            }
            return this.nextPageUrl;
        }
    }

    //查找下一页链接
    function findNextPageUrl(doc, reg) {
        reg = new RegExp(reg, 'i');
		var regHref = new RegExp('^javascript');
		var pageTagA = doc.getElementsByTagName('a');
		var item;
		if(pageTagA && pageTagA.length>0) {
			for(var i=0; i<pageTagA.length; i++) {
				item = pageTagA[i];
				if( reg.test(item.innerHTML.toLowerCase()) && !regHref.test(item.href) ) {
					return item.href;
				}
			}
		}
	}

    //查找下一页链接针对CNN特殊处理
    function findNextPageUrlSpecial1() {
        var itemLink
        var allLink = document.querySelectorAll('.dotnav > a');
        for (var i = 0; i < allLink.length; i++) {
            itemLink = allLink[i];
            if (/on/i.test(itemLink.className) && (i + 1) < allLink.length) {
                return allLink[i + 1].href;
            }
        }
        return null;
    }

    // 判断webUrl是否是需要匹配兼容网站
    function isWebUrlList(url) {
        var reg;
        var urlList = [
            '3g\\.163\\.com',
            'book.*\\.sina\\.cn',
            '^(?!blog).*?\.sina\.cn',
            'ebook.*\\.3g\\.qq\\.com',
            '.+\\.3g\\.qq\\.com',
            'wap\\.book\\.ifeng\\.com',
            'i\\.ifeng\\.com',
            '.+\\.3g\\.cn',
            'read\\.3gycw\\.com',
            'wap\\.sohu\\.com',
            'm\\.sohu\\.com',
            '.+\\.hongxiu\\.com',
            'wap\\.people\\.com\\.cn',
            '3g\\.xinhuanet\\.com',
            'm\\.tiexue\\.net',
            'm\\.huanqiu\\.com',
            'qidian\\.cn',
            'wap\\.tadu\\.com',
            'wap\\.jjwxc\\.com',
            'm\\.yahoo\\.com',
            'www\\.businessinsider\\.com',
            'mobile\\.washingtonpost\\.com',
            'edition\\.cnn\\.com',
            'marquee\\.blogs\\.cnn\\.com',
            '.+\\.foxnews\\.mobi',
            'www\\.huffingtonpost\\.com',
            'm\\.bbc\\.co\\.uk',
            'www\\.bbc\\.co\\.uk',
            'm\\.techcrunch\\.com',
            'gigaom\\.com',
            'm\\.engadget\\.com',
            'm\\.espn\\.go\\.com',
            'www\\.macrumors\\.com',
            'mobile\\.nytimes\\.com',
            'blog\\.sina\\.cn'
        ];
        for (var i = 0; i < urlList.length; i++) {
            reg = new RegExp(urlList[i], 'i');
            if (reg.test(url)) {
                return i;
            }
        }
        return -1;
    }

    // 过滤掉style,class,onXXX事件代码
    function filterHtml(str) {
        str = str.replace(/style\s*=\s*[\'\"][^\'\"]*[\'\"]/gi, '');
        str = str.replace(/class\s*=\s*[\'\"][^\'\"]*[\'\"]/gi, '');
        str = str.replace(/on[a-z]+\s*=\s*[\'\"][^\'\"]*[\'\"]/gi, '');
        return str;
    }

    // 过滤掉script以后的代码(新浪)
    function filterHtmlScript(str) {
        str = str.replace(/<script[^>]*>.*(?=<\/script>)<\/script>/gi, '#mx3_sign#');
        str = str.substring(0, str.indexOf('#mx3_sign#'));
        return str;
    }

    /**
        @description 过滤掉HTML代码特定字符串
        @param {string} str 要过滤的HTML代码
        @param {string} reg 匹配的字符串
    */
    function matchStrToNull(str, reg) {
        reg = new RegExp(reg, 'gi');
        str = str.replace(reg, '');
        return str;
    }

    /**
        @description 过滤掉HTML代码一类或某个tag (注意同类tag嵌套情况下及贪婪模式慎用!)
        @param {string} str 要过滤的HTML代码
        @param {string} reg 匹配的tag
        @param {number} isEnd 0 非贪婪 1 贪婪
        @param {string} attr 匹配的tag的属性
    */
    function matchTagToNull(str, tag, isEnd, attr) {
        attr = attr || '';
        if (isEnd == 0) {
            reg = '<(' + tag + ')\\s*' + attr + '[^>]*>[\\s\\S]*?<\\/\\1>'
        }
        else {
            reg = '<(' + tag + ')\\s*' + attr + '[^>]*>[\\s\\S]*<\\/\\1>';
        }
        reg = new RegExp(reg, 'gi');
        str = str.replace(reg, '');
        return str;
    }

    /**
        @description 过滤掉HTML代码的一部分
        @param {string} str 要过滤的HTML代码
        @param {string} reg 匹配的关键词
        @param {number} isEnd 0 去掉匹配位置之前的 1 去掉匹配位置之后的
    */
    function matchSectionToNul(str, reg, path) {
        if (path == 0) {
            reg = '^[\\s\\S]*' + reg;
        }
        else {
            reg += '[\\s\\S]*$';
        }
        reg = new RegExp(reg, 'i');
        return str.replace(reg, '');
    }

    // 把HTMLElement的类数组转换成字符串
    function objToStr(obj) {
        var str = '';
        for (var i = 0, l = obj.length; i < l; i++) {
            var o = obj[i];
            str += o.innerHTML;
        }
        return str;
    }

})(window);



