/**
 @fileOverview
 整理代码，分析页面的逻辑提出
 @author liujia <liujia@maxthon.net>
 */




(function (window) {
 
 // 阅读器对象
 var _mxReader = null;
 // iframe的html源码
 var _mxReaderPageHTML = "";
 // 构建阅读页的页面对象
 var _mxReaderPage = null;
 var _mxReaderLang = null;
 var _mxReaderSetting = null;
 
 // 常用函数变量,为了兼容旧有代码
 var getRect = readerTool.getRect;
 var getStyle = readerTool.getStyle;
 var getFontIntSize = readerTool.getFontIntSize;
 var deleteBlank = readerTool.deleteBlank;
 var isVisible = readerTool.isVisible;
 var compareUrlHost = readerTool.compareUrlHost;
 var unicodeHit = readerTool.unicodeHit;
 var stringTail2Number = readerTool.stringTail2Number;
 var isLikelyUrl = readerTool.isLikelyUrl;
 var getDefaultScrollWidth = readerTool.getDefaultScrollWidth;
 var findSameParent = readerTool.findSameParent;
 
 window.reader = {
 // 入口
 maxthonReaderLaunch: maxthonReaderLaunch,
 getTitle: getTitle,
 getContent: getContent,
 getNextUrl: getNextUrl
 };
 
 function getTitle() {
 return _mxReaderPage.getArticleTitle();
 }
 
 function getContent() {
 return _mxReaderPage.getContent().innerHTML;
 }
 
 function getNextUrl() {
 return _mxReaderPage.findNextPageUrl();
 }
 
 /**
  @description 分析入口
  @param document
  @return true代表可以启动阅读模式
  */
 function maxthonReaderLaunch(doc) {
 //try{
 if(!isLikelyUrl())
 return false;
 
 _mxReaderPage = new PageConstructor(doc);
 if (!_mxReaderPage.getCandidate()) {
 return false;
 }
 
 return true;
 //} catch(e){
 	//alert(e);
 //}
 }
 
 /**
  @description 页面构造分析
  @param document
  @return true代表可以启动阅读模式
  */
 function PageConstructor(doc) {
 this.contentDocument = doc;
 // 候选元素
 this.candidate = null;
 this.articleTitle = null;
 this.articleTitleText = null;
 this.content = null;
 this.pageNumber = 1;
 this.nextPageUrl = null;
 
 // 返回候选元素
 this.getCandidate = function() {
 if (this.candidate != null)
 return this.candidate;
 
 var candidates = this.getCandidateList();
 // 算分并且降分排序
 candidates.sort(function (a, b) { return b.score() - a.score()});
 if (!candidates || !candidates.length)
 return null;
 
 var results = [];
 // 计算分后，走一遍过滤条件
 for(var i = 0; i < candidates.length; i++) {
 var item = candidates[i];
 
 if (item.condition0())
 continue;
 
 if (item.condition1())
 continue;
 
 if (item.condition2())
 continue;
 
 if (item.condition3())
 continue;
 
 results.push(item);
 }
 
 if(results.length) {
 this.candidate = results[0];
 // 当有一个以上结果时,会再判断,如果成立改取他们的共同最近父元素
 if(results.length > 1) {
 var sibling = results[1];
 if( Math.abs(getRect(this.candidate.element).width - getRect(sibling.element).width) < 50
    && this.candidate.condition1/sibling.condition1 < 1.5
    && this.candidate.element !== sibling.element.parentElement) {
 this.candidate.element = findSameParent(this.candidate.element, sibling.element);
 }
 }
 var cs = getStyle(this.candidate.element);
 this.candidate.directionIsNormal = cs ? cs.direction === "ltr" : true;
 }
 
 return this.candidate;
 };
 
 // 返回候选元素
 this.getCandidateElement = function () {
 return (candidate = this.getCandidate()) ? candidate.element : null;
 };
 
 // 返回候选元素的内容
 this.getContent = function() {
 if (this.content != null)
 return this.content;
 
 var candidateElem = this.getCandidateElement();
 if (!candidateElem)
 return this.content = null;
 
 this.content = candidateElem.cloneNode(true);
 this.content = this.grabContent(candidateElem, this.content);
 
 return this.content;
 };
 
 // 获取候选列表
 this.getCandidateList = function () {
 // 设置死亡时间1秒
 var deadTime = Date.now() + 1E3;
 var allElements = this.contentDocument.getElementsByTagName("*");
 // 匹配元素列表
 var okMaybeItsACandidate = { BODY: 1, DIV: 1, TD: 1, P: 1, PRE: 1, DD: 1, SPAN: 1, STRONG: 1};
 var reList = [];
 
 for (var i = 0; i < allElements.length; i++) {
 var tag = allElements[i];
 if (okMaybeItsACandidate[tag.tagName]) {
 var item = this.createCandidateByTag(tag, this.contentDocument);
 if(item) reList.push(item);
 
 if (Date.now() > deadTime) {
 reList = [];
 break;
 }
 }
 }
 return reList;
 };
 
 // 根据原始类型获取候选者
 this.createCandidateByTag = function (tag, page) {
 if (!isVisible(tag, false))
 return null;
 
 if (getRect(tag).width < 450 || getRect(tag).height < 200)
 return null;
 
 // class和id淘汰列表
 var unlikelyCandidatesReg = /combx|comment|community|disqus|extra|foot|header|banner|menu|reply|talkback|remark|rss|shoutbox|sponsor|ad-break|agegate|pagination|pager|popup|tweet|twitter/i;
 for(var node = tag; node.tagName !== "BODY"; node = node.parentNode) {
 var unlikelyMatchString = node.className + node.id;
 if (unlikelyCandidatesReg.test(unlikelyMatchString)) {
 return null;
 }
 }
 
 return new ContentCandidate(tag, page);
 };
 
 // 返回文字title
 this.getArticleTitle = function () {
 if (this.articleTitleText != null)
 return this.articleTitleText;
 
 var maybeItsATitleRegex = /header|title|headline/i;
 var candidateElement = this.getCandidateElement();
 var candidateElementRect = getRect(candidateElement);
 var baseX = candidateElementRect.offsetLeft + candidateElementRect.width / 2;
 var baseY = candidateElementRect.offsetTop;
 
 var node = this.contentDocument.elementFromPoint(baseX, baseY - 150);
 // 如果元素的类型是img，重新计算x/y
 if (node && node.tagName === "IMG") {
 var nodeRect = getRect(node);
 if (nodeRect.height >= 300 && nodeRect.width >= candidateElementRect.width * 0.5)
 baseY = nodeRect.offsetTop;
 }
 
 var origTitle = this.contentDocument.title.toLowerCase();
 var candidateElementStyle = getStyle(candidateElement);
 var okMaybeTitleList = this.contentDocument.querySelectorAll("h1, h2, h3, h4, h5, .headline, .article_title, #hn-headline");
 var dstTitle = null;
 for (var i = 0; i < okMaybeTitleList.length; i++) {
 var curTitle = okMaybeTitleList[i];
 var curTitleText = curTitle.innerText;
 if (curTitleText.length < 4)
 continue;
 
 var titleRect = getRect(curTitle);
 var curTitleX = titleRect.offsetLeft + titleRect.width / 2;
 var deltaX = curTitleX - baseX;
 var deltaY = titleRect.offsetTop - baseY;
 var hypotenuse = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
 
 var curScore = Math.max(500 - hypotenuse, 0);
 if (!(hypotenuse > 500) &&
     !(titleRect.width < candidateElementRect.width * 0.5) &&
     !(curTitleX < candidateElementRect.left || curTitleX > candidateElementRect.right))
 {
 var fontSize = getFontIntSize(curTitle);
 if (fontSize < 12)
 continue;
 
 curScore *= fontSize / 16;
 if (fontSize > parseInt(candidateElementStyle.fontSize, 10) * 1.4)
 curScore *= 1.25;
 if (maybeItsATitleRegex.test(curTitle.className+curTitle.id))
 curScore *= 1.5;
 
 if (origTitle.length > 0) {
 var curText = curTitleText.toLowerCase();
 var diffLen = 0;
 
 if (origTitle.length == 0)
 diffLen = curText.length;
 else if (curText.length == 0)
 diffLen = origTitle.length;
 else {
 try {
 var reg = new RegExp("(?=.*?)[^" + origTitle +"](?=.*?)|(?=.*?)[^" + curText + "](?=.*?)", "g");
 var matchResult = null;
 while ((matchResult = reg.exec(origTitle + curText)) != null)
 diffLen += matchResult.length;
 }
 catch(err) {
 console.log("Invalid regular expression!");
 }
 }
 var addonScore = 1 - diffLen / origTitle.length;
 addonScore *= curScore * 0.2;
 curScore += addonScore;
 }
 if (!dstTitle || curScore > dstTitle.score) {
 dstTitle = curTitle;
 dstTitle.score = curScore;
 dstTitle.text = curTitleText;
 }
 }
 }
 
 if (dstTitle) {
 this.articleTitleText = dstTitle.text;
 this.articleTitle = dstTitle;
 }
 if (!this.articleTitleText) {
 this.articleTitleText = this.contentDocument.title;
 this.articleTitle = null;
 }
 return this.articleTitleText;
 };
 
 /**
  @description 根据传入的页面元素，获取它的页面位置
  @param
  0 原有页面的元素
  1 clone出来的元素
  */
 this.grabContent = function (candidateElement, contentElement) {
 function e(R) {
 if (q) q += R;
 if (x) x += R;
 if (p) p += R;
 if (s) s += R;
 }
 
 var needDeleteAttributes = /class|style|align/i;
 var neglectTags = { FORM: 1, IFRAME: 1, SCRIPT: 1, STYLE: 1, LINK: 1, INPUT: 1, SELECT: 1 };
 var maybeDeleteTags = { DIV: 1, TABLE: 1, OBJECT: 1, UL: 1 };
 var keepFontStyleTags = { I: 1, EM: 1 };
 var keepFontWeightTags = { B: 1, STRONG: 1, H1: 1, H2: 1, H3: 1, H4: 1, H5: 1, H6: 1};
 
 var pageHost = this.contentDocument.location.host;
 var maybeDeleteNodes = [];
 var node = candidateElement;
 var needCutNode = contentElement;
 var q = 0, x = 0, p = 0, s = 0;
 
 this.getArticleTitle();
 
 while (node && needCutNode) {
 var cutNode = null;
 var nodeStyle = null;
 var tagname = needCutNode.tagName;
 
 needCutNode.primalNode = node;
 
 if (tagname in neglectTags)
 cutNode = needCutNode;
 else if (node === this.articleTitle)
 cutNode = needCutNode;
 
 if (!cutNode) {
 nodeStyle = getStyle(node);
 if (tagname !== "PARAM" && nodeStyle.display === "none")
 cutNode = needCutNode;
 }
 
 if (!cutNode && needCutNode.childElementCount === 1) {
 var n;
 for (n = needCutNode.firstElementChild; n.childElementCount === 1; n = n.firstElementChild);
 if (n.tagName === "BR" && !/\S/.test(needCutNode.innerText))
 cutNode = needCutNode;
 else if (n.tagName === "SCRIPT")
 cutNode = needCutNode;
 }
 
 if (!cutNode) {
 if (/comment|photo-list/i.test(needCutNode.id + " " + needCutNode.className))
 cutNode = needCutNode;
 if (!cutNode && pageHost.substr(pageHost.length - 13, 13) == "wikipedia.org" && needCutNode.className == "editsection")
 cutNode = needCutNode;
 }
 
 if (!cutNode) {
 // 将没有块级子元素的DIV转成P
 if (tagname === "DIV" && needCutNode.parentNode) {
 var blockChilds = node.querySelectorAll("a, blockquote, dl, div, img, ol, p, pre, table, ul");
 if (!(q || nodeStyle["float"] !== "none") && !blockChilds.length) {
 var newNode = this.contentDocument.createElement("p");
 newNode.innerHTML = needCutNode.innerHTML;
 needCutNode.parentNode.replaceChild(newNode, needCutNode);
 needCutNode = newNode;
 needCutNode.primalNode = node;
 tagname = needCutNode.tagName;
 }
 }
 
 if(needCutNode.parentNode && tagname in maybeDeleteTags)
 maybeDeleteNodes.push(needCutNode);
 
 if (node !== candidateElement &&
     !q &&
     tagname !== "IMG" &&
     nodeStyle["float"] !== "none" &&
     (getRect(node).height >= 40 || node.childElementCount > 1))
 {
 q = 1;
 }
 
 var attrs = needCutNode.attributes;
 for (var ai = 0; ai < attrs.length; ai++) {
 var attrValue = attrs[ai].nodeName;
 if ( needDeleteAttributes.test(attrValue) ||
     (attrValue == "align" && /justify/i.test(attrs[ai].nodeValue)))
 {
 needCutNode.removeAttribute(attrValue);
 ai--;
 }
 }
 
 if (!p && nodeStyle.fontStyle !== "normal") {
 if (!(tagname in keepFontStyleTags))
 needCutNode.style.fontStyle = nodeStyle.fontStyle;
 p = 1;
 }
 if (!s && nodeStyle.fontWeight !== "normal") {
 if (!(tagname in keepFontWeightTags))
 needCutNode.style.fontWeight = nodeStyle.fontWeight;
 s = 1;
 }
 
 if (q) {
 if (q === 1 && getRect(node).width === getRect(candidateElement).width)
 needCutNode.setAttribute("class", "float full-width");
 else {
 needCutNode.setAttribute("class", "float " + nodeStyle["float"]);
 needCutNode.style.clear = nodeStyle.clear;
 }
 
 if (D = node.style.getPropertyValue("width"))
 needCutNode.style.width = D;
 else if (I = window.getMatchedCSSRules(node, "", true)) {
 for (j = I.length - 1; j >= 0; j--) {
 if (D = I[j].style.getPropertyValue("width")) {
 needCutNode.style.width = D;
 break;
 }
 }
 }
 
 if (q === 1 && !D) {
 needCutNode.style.width = getRect(node).width + "px";
 needCutNode.style.height = getRect(node).height + "px";
 }
 }
 
 if (tagname === "TABLE") {
 x || (x = 1);
 }
 else if (tagname === "IMG") {
 needCutNode.removeAttribute("border");
 needCutNode.removeAttribute("hspace");
 needCutNode.removeAttribute("vspace");
 needCutNode.removeAttribute("align");
 if (q) {
 needCutNode.style["float"] = nodeStyle["float"];
 needCutNode.style.clear = nodeStyle.clear;
 } else {
 var imgRect = getRect(node);
 if (imgRect.width < 128 && imgRect.height < 128)
 needCutNode.setAttribute("class", "tinyImage");
 if (nodeStyle["float"] !== "none") {
 needCutNode.setAttribute("class", "float " + nodeStyle["float"]);
 needCutNode.style["float"] = nodeStyle["float"];
 needCutNode.style.clear = nodeStyle.clear;
 }
 }
 var originalValue = needCutNode.getAttribute('original');
 if(originalValue)
 needCutNode.setAttribute("src", originalValue);
 else
 needCutNode.setAttribute("src", needCutNode.src);
 needCutNode.primalNode.node = needCutNode;
 needCutNode.primalNode.addEventListener("load", this.imgLoadDone, false);
 } else if (tagname === "FONT") {
 needCutNode.removeAttribute("size");
 needCutNode.removeAttribute("face");
 needCutNode.removeAttribute("color");
 
 if(/\b(4yt.net|jjwxc.net)\b/.test(pageHost))
 needCutNode.style.display = "none";
 } else if (tagname === "A" && needCutNode.parentNode) {
 var hrefText = needCutNode.getAttribute("href");
 if (hrefText && hrefText.length && (hrefText[0] === "#" || hrefText.substring(0, 11) === "javascript:")) {
 if (!x &&
     !needCutNode.childElementCount &&
     needCutNode.parentElement.childElementCount === 1)
 {
 this.contentDocument.evaluate("text()", needCutNode.parentElement, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null).snapshotLength || (cutNode = needCutNode);
 }
 
 if (!cutNode) {
 var newNode = this.contentDocument.createElement("span");
 if (needCutNode.childElementCount === 1 &&
     needCutNode.firstElementChild.tagName === "IMG") {
 var t = needCutNode.firstElementChild;
 if( t.width > 32 && t.height > 32 )
 newNode.setAttribute("class", "image-anchor");
 }
 newNode.className || newNode.setAttribute("class", "text-anchor");
 needCutNode.firstChild &&	newNode.appendChild(needCutNode.firstChild);
 needCutNode.parentNode.replaceChild(newNode, needCutNode);
 needCutNode = newNode;
 }
 }
 if (needCutNode.href) {
 needCutNode.setAttribute("href", needCutNode.href);
 needCutNode.setAttribute("target", "_new");
 }
 } else if (tagname === "SPAN") {
 if(pageHost == "html.hjsm.tom.com")
 needCutNode.style.display = "none";
 if (nodeStyle.display === "block")
 needCutNode.style.display = "block";
 }
 }
 
 if (j = cutNode ? null : node.firstElementChild) {
 node = j;
 needCutNode = needCutNode.firstElementChild;
 e(1);
 } else {
 for (var da; node !== candidateElement && !(da = node.nextElementSibling);) {
 node = node.parentElement;
 needCutNode = needCutNode.parentElement;
 e(-1);
 }
 if (node === candidateElement) {
 if(cutNode && cutNode.parentElement)
 cutNode.parentElement.removeChild(cutNode);
 break;
 }
 node = da;
 needCutNode = needCutNode.nextElementSibling;
 if (q === 1) q = 0;
 if (x === 1) x = 0;
 if (p === 1) p = 0;
 if (s === 1) s = 0;
 }
 
 if (cutNode) if (cutNode.parentElement) {
 do {
 j = cutNode.parentElement;
 j.removeChild(cutNode);
 cutNode = j;
 } while (j != node.parentElement && j.childElementCount == 0 && !/\S/.test(j.innerText))
 }
 }
 
 for (var di = maybeDeleteNodes.length - 1; di >= 0; di--) {
 var dn = maybeDeleteNodes[di];
 if (dn.parentNode && this.shouldDelete(dn, dn.primalNode, candidateElement)) {
 var deleteNode = dn;
 deleteNode.parentElement.removeChild(deleteNode);
 }
 }
 
 var floatElements = contentElement.querySelectorAll(".float");
 for (var fi = 0; fi < floatElements.length; fi++) {
 var floatNode = floatElements[fi];
 f = floatNode.querySelectorAll("a, span.image-anchor");
 h = floatNode.querySelectorAll("span.text-anchor");
 f = floatNode.parentNode && h.length > f.length;
 if (!f) {
 h = floatNode.querySelectorAll("embed, object").length;
 k = floatNode.primalNode.querySelectorAll("embed, object").length;
 if (!h && k) f = true;
 }
 f && floatNode.parentNode.removeChild(floatNode);
 }
 
 return contentElement;
 };
 
 this.imgLoadDone = function (evt)
 {
 var imgNode = evt.target;
 if (imgNode.primalNode && imgNode.primalNode.src != imgNode.src)
 imgNode.primalNode.src = imgNode.src;
 };
 
 this.shouldDelete = function (node, primalNode, candidateNode)
 {
 var positive = /article|body|content|entry|page|pagination|post|text/i;
 var negative = /comment|combx|disqus|contact|foot|link|masthead|media|meta|promo|related|scroll|shoutbox|sponsor|tags|widget|share|pagination/i;
 
 if (!node.parentElement)
 return false;
 
 // 没有子元素的节点
 if (node.tagName !== "OBJECT" && node.tagName !== "EMBED") {
 var  hasValidNode = false;
 for (var i = 0, len = node.childNodes.length; i < len; i++) {
 var n = node.childNodes[i],	t = n.nodeType;
 if (t === Node.ELEMENT_NODE || t === Node.TEXT_NODE && /\S/.test(n.nodeValue)) {
 hasValidNode = true;
 break;
 }
 }
 if (!hasValidNode) {
 return true;
 }
 }
 
 // id/class为消极属性的元素
 var icScore = 0;
 if (primalNode) {
 positive.test(primalNode.className) && icScore++;
 positive.test(primalNode.id) && icScore++;
 negative.test(primalNode.className) && icScore--;
 negative.test(primalNode.id) && icScore--;
 }
 if (icScore < 0) {
 return true;
 }
 
 if (node.tagName === "UL") {
 if (primalNode.querySelector("iframe") && primalNode.querySelector("script"))
 return true;
 return false;
 }
 
 // 插件元素
 if (node.tagName === "OBJECT") {
 var hostRegex = /youtube|vimeo|dailymotion|livestream\.com|viddler\.com/;
 var effectiveEmbed = node.querySelector("embed[src]")
 if (hostRegex.test(effectiveEmbed.src))
 return false;
 
 var data = node.getAttribute("data")
 if (hostRegex.test(data))
 return false;
 
 var pageUrl = this.contentDocument.location.href;
 if (compareUrlHost(pageUrl, effectiveEmbed.src) || compareUrlHost(pageUrl, data))
 return false;
 
 return true;
 }
 
 // 文字链接元素
 if (primalNode.childElementCount === 1) {
 var n = primalNode.firstElementChild;
 if (n.tagName === "A" && n.getElementsByTagName("img").length == 0) {
 if (getFontIntSize(n) / getFontIntSize(candidateNode) < 0.75)
 return true;
 }
 }
 if (node.childElementCount === 1) {
 var n = node.firstElementChild;
 if (n.tagName === "A")
 return false;
 }
 
 // 图片元素
 var imgs = node.querySelectorAll("img"), imgsLen = imgs.length;
 var tinyImgNum = 0;
 if (imgsLen) {
 var imgsArea = 0;
 for (var i = 0; i < imgsLen; i++) {
 var n = imgs[i].primalNode;
 if (isVisible(n, false)) {
 var r = getRect(n);
 imgsArea += (r.width / imgsLen) * (r.height / imgsLen);
 var area = r.width * r.height;
 if (!compareUrlHost(this.contentDocument.location.href, n.src) || area < 16384)
 tinyImgNum++;
 }
 }
 if (imgsArea > 4E4)
 return false;
 }
 
 // 段落元素
 if (!this.isLittleParagraph(node, 10))
 return false;
 var pNum = node.querySelectorAll("p").length;
 var brNum = node.querySelectorAll("br").length;
 pNum = pNum + Math.floor(brNum / 2);
 if (tinyImgNum > pNum) {
 return true;
 }
 
 // 其他
 if (primalNode.querySelectorAll("li a[href]").length > pNum) {
 return true;
 }
 if (node.querySelectorAll("input").length / pNum > 0.33) {
 return true;
 }
 if (node.textContent.trim().length < 25 && (imgsLen == 0 || imgsLen > 1 && tinyImgNum > 0)){
 return true;
 }
 if (node.querySelector("embed")) {
 return true;
 }
 
 // 链接文字占比
 var ratio = this.anchorTextRatio(node);
 if (icScore >= 1 && ratio > 0.5) {
 return true;
 }
 if (icScore < 1 && ratio > 0.2) {
 return true;
 }
 
 // 表格
 if (node.tagName === "TABLE") {
 var len1 = deleteBlank(node.innerText).length;
 var len2 = deleteBlank(primalNode.innerText).length;
 if (len1 <= len2 * 0.5) {
 return true;
 }
 }
 
 return false;
 };
 
 this.isLittleParagraph = function (node, c)
 {
 var e = 0, d = node.textContent;
 for (var i = -1; e < c && (i = d.indexOf(",", i + 1)) >= 0;)
 e++;
 return e < c;
 };
 
 this.anchorTextRatio = function (node)
 {
 var textLen = deleteBlank(node.textContent).length;
 if (!textLen) return 0;
 
 var linkNodes = node.querySelectorAll("a");
 var linkTextLen = 0;
 for (var i = 0; i < linkNodes.length; i++) {
 var linkNode = linkNodes[i];
 if(linkNode.href)
 linkTextLen += deleteBlank(linkNode.textContent).length;
 }
 return linkTextLen / textLen;
 };
 
 this.findNextPageUrl = function ()
 {
 if (!this.candidate)
 return null;
 
 if(this.nextPageUrl)
 return this.nextPageUrl;
 
 // maxthon.cu.17k.com下一页特殊处理
 if(this.contentDocument.location.host == "maxthon.cu.17k.com") {
 this.nextPageUrl = findNextPageUrlSpecial1(this.contentDocument.location.href);
 return this.nextPageUrl;
 }
 
 // read.dangdang.com下一页特殊处理
 if(this.contentDocument.location.host == "read.dangdang.com") {
 this.nextPageUrl = findNextPageUrlSpecial2(this.contentDocument);
 return this.nextPageUrl;
 }
 
 // html.hjsm.tom.com下一页特殊处理
 if(this.contentDocument.location.host == "html.hjsm.tom.com") {
 this.nextPageUrl = findNextPageUrlSpecial3(this.contentDocument);
 return this.nextPageUrl;
 }
 
 for(var p = this.candidate.element; p.nodeType == Node.ELEMENT_NODE && p.tagName != "HTML";) {
 //console.log("----------------findNextPageUrl() dstElement:" + p.className);
 if (this.nextPageUrl = this.getNextPageUrl(p))
 return this.nextPageUrl;
 p = p.parentElement;
 }
 
 //console.log("reader mode error msg: ");
 return this.nextPageUrl;
 };
 
 this.getNextPageUrl = function (elem)
 {
 var result = null;
 var maxLinkScore = 0;
 var allLink = this.contentDocument.evaluate("//a[@href]", elem, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
 for (var i = 0; i < allLink.snapshotLength; i++) {
 var itemLink = allLink.snapshotItem(i);
 var linkScore = this.calcNextPageLinkScore(itemLink);
 // console.log(itemLink.href + "  linkScore:" + linkScore);
 if (linkScore > maxLinkScore) {
 result = itemLink.href;
 maxLinkScore = linkScore;
 }
 }
 return result;
 }
 
 this.calcNextPageLinkScore = function (linkNode)
 {
 function compareUrlPageIndex(currentIndex, nextIndex, linkText, pageNum) {
 // console.log("currentIndex="+currentIndex+"    nextIndex="+nextIndex+"    linkText="+linkText+"    pageNumber="+pageNum);
 // 第一种情况: link text
 if (/下一页|下一章|下一篇|下一节|下页/i.test(linkText))
 return true;
 else if(linkText.length > 10)
 return false;
 
 // 第二种情况：nextUrl = currentUrl+1
 if (nextIndex.substring(0, currentIndex.length) == currentIndex) {
 nextIndex = nextIndex.substring(currentIndex.length);
 currentIndex = "";
 }
 var nextIndexNumber = stringTail2Number(nextIndex);
 if (isNaN(nextIndexNumber))
 return false;
 var currentIndexNumber = currentIndex ? stringTail2Number(currentIndex) : NaN;
 if (isNaN(currentIndexNumber) /*|| currentIndexNumber >= 100*/)
 currentIndexNumber = pageNum;
 if (nextIndexNumber == currentIndexNumber) {
 if(stringTail2Number(linkText) == currentIndexNumber + 1)
 return true;
 }
 if(nextIndexNumber == currentIndexNumber + 1)
 return true;
 
 // 第三种情况: nextUrl > currentUrl+1
 if(nextIndexNumber > currentIndexNumber+1 && parseInt(linkText, 10) == pageNum + 1)
 return true;
 
 return false;
 }
 
 function searchParse(buffer) {
 var result = {};
 var pairList = buffer.substring(1).split("&");
 for (var i = 0; i < pairList.length; i++) {
 var pair = pairList[i],	value = pair.indexOf("=");
 if (value == -1)
 result[pair] = null;
 else
 result[pair.substring(0, value)] = pair.substring(value + 1);
 }
 return result;
 }
 var pageUrl = this.contentDocument.location;
 var linkText = linkNode.textContent;
 if (linkNode.host != pageUrl.host) return 0;
 if (linkNode.pathname == pageUrl.pathname && linkNode.search == pageUrl.search) return 0;
 if (linkNode.toString().indexOf("#") != -1) return 0;
 if (!isVisible(linkNode, false)) return 0;
 
 // 位置比较
 var linkRect = getRect(linkNode), candidateRect = getRect(this.candidate.element);
 var distanceV = Math.max(0, Math.max(candidateRect.top - (linkRect.top + linkRect.height), linkRect.top - (candidateRect.top + candidateRect.height)));
 if (distanceV > 450) return 0;
 var distanceH = Math.max(-1, Math.max(candidateRect.left - (linkRect.left + linkRect.width), linkRect.left - (candidateRect.left + candidateRect.width)));
 if (distanceH > -1) return 0;
 
 // 路径比较
 var linkPathName = linkNode.pathname.substring(1).split("/");
 linkPathName[linkPathName.length - 1] || linkPathName.pop();
 var pagePathName = pageUrl.pathname.substring(1).split("/");
 pagePathName[pagePathName.length - 1] || pagePathName.pop();
 if(!pagePathName[0]) pagePathName.shift();
 if (linkPathName.length < pagePathName.length)
 return 0;
 // console.log("linkPathName:" + linkPathName);
 
 // 路径索引比较
 var baseScore = 0;
 var index = linkPathName.length - 1;
 var linkIndex = linkPathName[index];
 var pageIndex = pagePathName[index] ? pagePathName[index] : "";
 // console.log("pageIndex:" + pageIndex + "  linkIndex:" + linkIndex);
 if (pageIndex != linkIndex) {
 // 去掉相同的后缀
 if (linkIndex.length >= pageIndex.length) {
 var postfix = 0;
 for (; linkIndex[linkIndex.length - 1 - postfix] == pageIndex[pageIndex.length - 1 - postfix];)
 postfix++;
 if (postfix) {
 linkIndex = linkIndex.substring(0, linkIndex.length - postfix);
 pageIndex = pageIndex.substring(0, pageIndex.length - postfix)
 }
 }
 if (compareUrlPageIndex(pageIndex, linkIndex, linkText, this.pageNumber))
 baseScore = Math.pow(1.5, index - linkPathName.length + 1);
 else
 return 0;
 }
 // console.log("path done. baseScore:" + baseScore);
 
 // search比较
 var  k = 0;
 if (linkNode.search || pageUrl.search) {
 baseScore = 0;
 linkSearches = searchParse(linkNode.search);
 pageSearches = searchParse(pageUrl.search);
 //console.log("----linkNode.search:" + linkNode.search + "    pageUrl.search:"+pageUrl.search);
 for (var key in pageSearches) {
 var pageIndex = pageSearches[key];
 var linkIndex = key in linkSearches ? linkSearches[key] : null;
 //console.log("key:" + key + "  pageIndex:" + pageIndex +"  linkIndex:" + linkIndex);
 if(!linkIndex) {
 baseScore = 0;
 break;
 }
 if (pageIndex != linkIndex) {
 if (pageIndex == null) pageIndex = "";
 if (linkIndex == null) linkIndex = "";
 
 if (linkIndex.length < pageIndex.length)
 k++;
 else if (compareUrlPageIndex(pageIndex, linkIndex, linkText, this.pageNumber))
 baseScore = Math.max(baseScore, 1 / 1.5);
 else
 k++;
 }
 }
 }
 // console.log("search done. baseScore" + baseScore);
 
 if (!baseScore)
 return 0;
 
 var result = 10 * (Math.pow(2, -k) + baseScore) + 5 * distanceV / 225;
 if (linkNode.parentNode.tagName == "LI") result += 3;
 h = linkNode.innerText;
 if (/next|>|&gt;|&#62;|\u00bb|&raquo;|&#187;/i.test(h)) result += 5;
 if (/page.*\d+/i.test(h)) result += 5;
 if (/continue|read\s+more/i.test(h)) result += 3;
 if (/下一页|下一章/i.test(h)) result += 5;
 return result;
 }
 }
 
 /**
  @description 生成内容候选者
  @param
  0 元素对象
  1 document
  */
 function ContentCandidate(tag, page) {
 this.element = tag;
 this.contentDocument = page;
 this.directionIsNormal = true;
 this.condition1 = 0;
 
 // 内容总评分
 this.getContentScore = function () {
 var score = 0, texts = this.textList;
 for (var i = 0; i < texts.length; i++)
 score += this.calcTextScore(texts[i]);
 return score;
 };
 
 // 具体评分
 this.calcTextScore = function (text) {
 if (!text) return 0;
 var textLen = text.length;
 if(textLen < 1)
 return 0;
 
 var delta = 1;
 // 文字所在层级越深分越低
 for (var node = text.parentNode; node && node != this.element;) {
 delta -= 0.1;
 node = node.parentNode;
 }
 return Math.pow(textLen * delta, 1.25);
 };
 
 // class的权重
 this.getClassWeight = function () {
 // 加分项
 var plusWeight = /article|entry|post|body|column|main|content/i;
 // 减分项
 var minusWeight = /comment|combx|disqus|foot|header|list|item|menu|rss|shoutbox|sidebar|sponsor/i;
 var delta = 1;
 
 for (var node = this.element; node; node = node.parentElement) {
 var matchString = node.className + node.id;
 if (matchString) {
 if (plusWeight.test(matchString)) delta += 0.4;
 if (minusWeight.test(matchString)) delta -= 0.8
 }
 }
 return delta;
 };
 
 // 编码类型
 this.getEncodingType = function () {
 if (this.encodingType != 0)
 return;
 
 if (this.textList && this.textList.length > 0) {
 var len = Math.min(this.textList.length, 3);
 for(var i = 0; i < len; i++) {
 var textChars = this.textList[i].nodeValue.trim();
 var textLen = Math.min(textChars.length, 10);
 for (var j = 0; j < textLen; j++) {
 if(unicodeHit(textChars.charCodeAt(j))){
 this.encodingType = 3;
 return;
 }
 }
 }
 }
 this.encodingType = 1;
 };
 
 // 内容分 * class权重
 this.score = function () {
 return this.contentScore * this.classWeight;
 };
 
 // 根据元素获取文字列表
 this.getTextList = function (tag) {
 var result = [];
 
 var node = tag;
 // 特殊判断
 if(node.firstElementChild && node.firstElementChild.tagName === "FOUNDER-CONTENT")
 node = tag.firstElementChild;
 
 // 淘汰列表
 var irrespectiveTag = {A: 1, DT: 1, OL: 1, OPTION: 1, SCRIPT: 1, NOSCRIPT: 1, STYLE: 1, UL: 1, IFRAME: 1, TEXTAREA: 1, };
 var allText = this.contentDocument.evaluate("text()|*/text()|*/li/text()|*/strong/text()|*/span/text()|*/span/*/text()|*/font/text()|*/font/*/text()|blockquote/text()|blockquote/*/text()|*/blockquote/text()|*/blockquote/*/text()|*/div/p/text()|*/div/p/index/text()|*/p/text()", node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
 
 for (var i = 0; i < allText.snapshotLength; i++) {
 var textNode = allText.snapshotItem(i);
 // 不在淘汰条件里且内容不为空
 if (!irrespectiveTag[textNode.parentNode.tagName] && /\S/.test(textNode.nodeValue)) {
 result.push(textNode);
 }
 }
 return result;
 };
 
 // 得分 * 编码类型小于100,放弃
 this.condition0 = function() {
 this.getEncodingType();
 var condition = this.score() * this.encodingType;
 if(condition < 100)
 return true;
 
 return false;
 };
 
 // 计算区域占比小于5,放弃
 this.condition1 = function () {
 var contentArea = getRect(this.element).width * getRect(this.element).height;
 var invalidArea = 0;
 
 // 将插件面积加入无效区域
 var objectList = this.element.querySelectorAll("object");
 for (var i = 0; i < objectList.length; i++) {
 var o = objectList[i];
 invalidArea += o.clientWidth * o.clientHeight;
 }
 
 // 将大图片加入无效区域
 var invalidImgSize = contentArea * 0.3;
 var invalidImgWidth = getRect(this.element).width * 0.2;
 var imgList = this.element.querySelectorAll("img");
 for (var i = 0; i < imgList.length; i++) {
 var r = getRect(imgList[i]);
 if (r.width >= invalidImgWidth && r.height > 50) {
 var imgArea = r.width * r.height;
 if (imgArea < invalidImgSize)
 invalidArea += imgArea;
 }
 }
 
 // 加入其他无效区域
 var others = this.contentDocument.evaluate("*[name()!='img' and name()!='object']", this.element, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
 if (others.snapshotLength > 2) for (var i = 0; i < others.snapshotLength; i++) {
 var item = others.snapshotItem(i);
 if (/comment/i.test(item.id + " " + item.className) || !/\S/.test(item.innerText) && item.getElementsByTagName("img").length == 0) {
 if(item.tagName == "BR") {
 var a = parseInt(getStyle(this.element).lineHeight, 10) * parseInt(getStyle(this.element).width, 10);
 if (a > 0)
 invalidArea += a;
 }	else {
 var r = getRect(item);
 var a = r.width * r.height;
 if (a > 0)
 invalidArea += a;
 }
 }
 }
 
 // 计算平均字体大小
 var averageFontsize = 16;
 var fontSizeSum = fontSizeCount = 0;
 for (var i = 0; i < this.textList.length; i++) {
 if (pNode = this.textList[i].parentNode) {
 fontSizeSum += getFontIntSize(pNode);
 fontSizeCount++;
 }
 }
 if (fontSizeCount)
 averageFontsize = fontSizeSum / fontSizeCount;
 
 var score = this.score();
 var textsArea = contentArea - invalidArea;
 this.getEncodingType();
 
 this.condition1 = score / textsArea * 1E3 * (averageFontsize / 16) * this.encodingType;
 
 if (this.condition1 < 5)
 return true;
 return false;
 };
 
 // 判断hr分割面积,小于180,放弃
 this.condition2 = function () {
 var hrTags = this.element.querySelectorAll("hr");
 var count = 0;
 var rect = getRect(this.element)
 var comparedWidth = rect.width * 0.7;
 
 for (var i = 0; i < hrTags.length; i++)
 hrTags[i].clientWidth > comparedWidth && count++;
 
 if (count)
 if (rect.height / count < 180) return true;
 
 return false;
 };
 
 // a链接判断,比值小于300,放弃
 this.condition3 = function () {
 var titleAnchors = this.contentDocument.evaluate("(h1|h2|h3|h4|h5|h6|*/h1|*/h2|*/h3|*/h4|*/h5|*/h6)[a]", this.element, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
 if (titleAnchors.snapshotLength > 2) {
 var count = 0;
 var rect = getRect(this.element);
 var comparedHeight = rect.height * 0.05;
 for (var i = 0; i < titleAnchors.snapshotLength; i++) {
 var item = titleAnchors.snapshotItem(i);
 var anchorRect = getRect(item);
 if(anchorRect.top - rect.top > comparedHeight && rect.bottom - anchorRect.bottom > comparedHeight)
 count++;
 }
 if (rect.height / count < 300)
 return true;
 }
 return false;
 };
 
 this.textList = this.getTextList(tag);
 this.contentScore = this.getContentScore();
 this.classWeight = this.getClassWeight();
 this.encodingType = 0;
 }
 
 /**
  * 特殊下一页url获取
  */
 // 1. maxthon.cu.17k.com
 function findNextPageUrlSpecial1(currentUrl)
 {
 var nextUrl = null;
 
 var baseUrl = currentUrl.split("_");
 var index = parseInt(baseUrl[1].split(".")[0], 10);
 for(var i=0; i < chapterIds.length; i++) {
 if(chapterIds[i] == index) {
 nextUrl = baseUrl[0] + "_" + chapterIds[i+1] + ".html";
 break;
 }
 }
 return nextUrl;
 }
 
 // 2. read.dangdang.com
 function findNextPageUrlSpecial2(currentDoc)
 {
 var nextUrl = null;
 var hrefText = null; // javascript:{gotoNextSection(2405826, 15872);}
 
 var allLink = currentDoc.evaluate("//a[@href]", currentDoc.body, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
 for (var i = 0; i < allLink.snapshotLength; i++) {
 var itemLink = allLink.snapshotItem(i);
 if(itemLink.textContent == "下一页>>") {
 hrefText = itemLink.href;
 break;
 }
 }
 // console.log("hrefText: " + hrefText);
 
 var regex = /\b(\d+)\b/g;
 var arrMactches = hrefText.match(regex)
 var secId = arrMactches[0];
 var bookId = arrMactches[1];
 // console.log("secId: " + secId + "  bookId: " + bookId);
 
 nextUrl = "http://read.dangdang.com/content_" + secId + "?ref=read-3-C&book_id=" + bookId;
 return nextUrl;
 }
 
 // 3. html.hjsm.tom.com
 function findNextPageUrlSpecial3(currentDoc)
 {
 function fnSniffer() {
 var sUrl = currentDoc.location.href;
 var iIdx = sUrl.lastIndexOf("/");
 
 if (-1 == sUrl.indexOf("&chapter_id=")) {
 sUrl = sUrl.substring(iIdx + 5);
 iIdx = sUrl.indexOf(",");
 sUrl = '&chapter_id=' + parseInt(sUrl.substring(0, iIdx), 16);
 } else {
 sUrl = sUrl.substr(sUrl.indexOf("&chapter_id="));
 }
 
 for (var ii = 0, jj = $TOC.length; ii < jj; ii++) {
 if (-1 != $TOC[ii].indexOf(sUrl)) {
 return (ii + 1);
 }
 }
 
 var sUrl = currentDoc.location.href;
 var checkurl = sUrl.split("#");
 if(checkurl.length!='1')
 sUrl = sUrl.substring(0,   checkurl[0].length);
 
 var checkurl2 = sUrl.split("?");
 if(checkurl2.length!='1')
 sUrl = sUrl.substring(0,   checkurl2[0].length);
 
 for (var ii = 0, jj = $TOC.length; ii < jj; ii++) {
 if(sUrl == $TOC[ii]){
 return (ii + 1);
 }
 }
 }
 
 if(currentDoc.getElementById('lastMsg') != null)
 return null;
 
 if(nextpage)
 return nextpage;
 
 var iIdx = fnSniffer();
 if (iIdx == $TOC.length)
 return null;
 
 return $TOC[iIdx];
 }
 })(window);
