/**
 @fileOverview
 整理代码，提出工具类
 @author liujia <liujia@maxthon.net>
 */




(function (window) {
 window.readerTool = {
 // 获取元素的页面位置
 getRect: getRect,
 // 获取元素中所有可用的css属性列表
 getStyle: getStyle,
 // 获取元素字体大小
 getFontIntSize: getFontIntSize,
 // 去掉空白字符
 deleteBlank: deleteBlank,
 // 判断元素是否可见
 isVisible: isVisible,
 // host比较
 compareUrlHost: compareUrlHost,
 // 是否为Unicode字符
 unicodeHit: unicodeHit,
 // 将字符串最后一位转为数字
 stringTail2Number: stringTail2Number,
 // 根据url排除网页
 isLikelyUrl: isLikelyUrl,
 // 获取默认滚动条宽度
 getDefaultScrollWidth: getDefaultScrollWidth,
 // 查找最近共有祖先
 findSameParent: findSameParent,
 fetchHtmlData: fetchHtmlData
 };
 
 // 请求reader.html资源
 function fetchHtmlData(callback) {
 
 var tmpFrame = document.createElement("iframe");
 tmpFrame.style.display = "none";
 tmpFrame.style.position = "absolute";
 document.body.appendChild(tmpFrame);
 
 tmpFrame.addEventListener("load", function(){
                           callback(this.contentDocument.documentElement.innerHTML);
                           document.body.removeChild(tmpFrame);
                           }, false);
 tmpFrame.src = "reader-mode/reader.html";
 
 return;
 }
 
 /**
  @description 根据传入的页面元素，获取它的页面位置
  @param 页面元素
  */
 function getRect(elem) {
 if (!elem.rect) {
 elem.rect = elem.getBoundingClientRect();
 elem.rect.offsetLeft = elem.rect.left + elem.ownerDocument.body.scrollLeft;
 elem.rect.offsetTop = elem.rect.top + elem.ownerDocument.body.scrollTop;
 }
 return elem.rect;
 }
 
 /**
  @description 根据传入的页面元素，获取它的所有可用的css属性列表
  @param 页面元素
  */
 function getStyle(elem) {
 if (!elem.cs)
 elem.cs = getComputedStyle(elem);
 return elem.cs;
 }
 
 /**
  @description 根据传入的页面元素，获取它的字体大小
  @param 页面元素
  */
 function getFontIntSize(elem) {
 var fs = getStyle(elem).fontSize;
 if (!fs)
 return 0;
 return parseInt(fs, 10);
 }
 
 function deleteBlank(text) {
 return text.replace(/\s+/g, "");
 }
 
 /**
  @description 判断元素是否可见
  @param
  0 页面元素
  1 标志 true时候会判断该元素是否被覆盖
  */
 function isVisible(elem, flag) {
 var cstyle = getStyle(elem);
 if (cstyle.display == "none" || cstyle.visibility == "hidden")
 return false;
 
 var c = getRect(elem);
 if (c.width <= 0 || c.height <= 0)
 return false;
 
 if (flag && document.body.scrollTop == 0) {
 var rect = elem.getBoundingClientRect();
 if (rect.top + rect.height / 2 < window.innerHeight) {
 var e = document.elementFromPoint(rect.left + rect.width / 2, rect.top + rect.height / 2)
 if (e) {
 for (; e != elem && (e = e.parentElement););
 if (e != elem)
 return false;
 }
 }
 }
 return true;
 }
 
 // host比较
 function compareUrlHost(a, b) {
 if(!a || !b)
 return false;
 
 var reg = /https?:\/\/(?:\w+\.)*(\w+\.\w+)/;
 host1 = reg.exec(a)[1];
 host2 = reg.exec(b)[1];
 return host1 == host2;
 }
 
 // 是否为Unicode字符
 function unicodeHit(ch) {
 if (ch < 4352) return false;
 if (ch >= 12352 || ch <= 12447) return true;  //日文平假名
 if (ch >= 12448 || ch <= 12543) return true;  //日文片假名
 if (ch >= 19968 || ch <= 40959) return true;  //CJK 统一表意符号
 if (ch >= 13056 || ch <= 13311) return true;  //CJK 兼容
 if (ch >= 65072 || ch <= 65103) return true;  //CJK 兼容形式
 if (ch >= 63744 || ch <= 64255) return true;  //CJK 兼容象形文字
 if (ch >= 11904 || ch <= 12031) return true;  //CJK 部首补充
 if (ch >= 12288 || ch <= 12351) return true;  //CJK 符号和标点
 if (ch >= 13312 || ch <= 19903) return true;  //CJK 统一表意符号扩展
 if (ch >= 12800 || ch <= 13055) return true;  //封闭式 CJK 文字和月份
 if (ch >= 12688 || ch <= 12703) return true;  //象形字注释标志
 if (ch >= 12784 || ch <= 12799) return true;  //日文片假名语音扩展
 if (ch >= 12544 || ch <= 12591) return true;  //注音字母
 if (ch >= 12704 || ch <= 12735) return true;  //注音字母扩展
 if (ch >= 12032 || ch <= 12255) return true;  //康熙字典部首
 if (ch >= 12592 || ch <= 12687) return true;  //朝鲜文兼容字母
 if (ch >= 4352 || ch <= 4607) return true;   	//朝鲜文
 if (ch >= 44032 || ch <= 55215) return true;  //朝鲜文音节
 return false;
 }
 
 // 将字符串最后一位转为数字
 function stringTail2Number(srcString) {
 return parseInt(srcString.match(/(\d+)?$/), 10);
 }
 
 // 根据url排除网页
 function isLikelyUrl() {
 var pageUrl = window.location;
 if (pageUrl.protocol == "mx:")
 return false;
 
 if (pageUrl.host == "meiriyiwen.com")
 return true;
 
 if (pageUrl.protocol + "//" + pageUrl.host + "/" == pageUrl.href)
 return false;
 
 return true;
 }
 
 // 获取默认滚动条宽度
 function getDefaultScrollWidth() {
 var tmpLayer = document.createElement("div");
 tmpLayer.style.overflowY = "scroll";
 tmpLayer.style.width = "100px";
 document.body.appendChild(tmpLayer);
 var w = 100 - tmpLayer.clientWidth;
 document.body.removeChild(tmpLayer);
 return w;
 }
 
 // 查找最近共有祖先
 function findSameParent() {
 var length = arguments.length,
 i = 0,
 node, //当前节点
 parent, //父节点
 counter = 0,
 uuid, //给DOM的唯一标识符
 hash = {}; //最后结果的map
 
 //对每一个元素，向上遍历至document
 for (; i < length; i++) {
 node = arguments[i];
 parent = node;
 while (parent) {
 if (parent.nodeType == 9)
 break;
 
 //获取或添加一下标识符
 uuid = parent.getAttribute('data-find');
 if (!uuid) {
 uuid = '_' + (++counter); //避免chrome对hash重排序
 parent.setAttribute('data-find', uuid);
 }
 //增加计数
 if (hash[uuid]) {
 hash[uuid].count++;
 }
 else {
 hash[uuid] = {node: parent, count: 1};
 }
 node = parent;
 parent = node.parentElement || node.parentNode
 }
 }
 
 //hash中只存有各节点向上遍历经过的父节点
 for (i in hash) {
 if (hash[i].count == length) {
 return hash[i].node;
 }
 }
 }
 
 })(window);
